.TH PTHREAD_DETACH 3 LinuxThreads

.SH NAME
pthread_detach \- put a running thread in the detached state

.SH SYNOPSIS
#include <pthread.h>

int pthread_detach(pthread_t th);

.SH DESCRIPTION
\fBpthread_detach\fP put the thread \fIth\fP in the detached state. This
guarantees that the memory resources consumed by \fIth\fP will be freed
immediately when \fIth\fP terminates. However, this prevents other threads
from synchronizing on the termination of \fIth\fP using \fBpthread_join\fP.

A thread can be created initially in the detached state, using the
\fBdetachstate\fP attribute to \fBpthread_create\fP(3). In contrast,
\fBpthread_detach\fP applies to threads created in the joinable state, and
which need to be put in the detached state later.

After \fBpthread_detach\fP completes, subsequent attempts to perform
\fBpthread_join\fP on \fIth\fP will fail. If another thread is already joining
the thread \fIth\fP at the time \fBpthread_detach\fP is called,
\fBpthread_detach\fP does nothing and leaves \fIth\fP in the joinable state.

.SH "RETURN VALUE"
On success, 0 is returned. On error, a non-zero error code is returned.

.SH ERRORS
.TP
\fBESRCH\fP
No thread could be found corresponding to that specified by \fIth\fP
.TP
\fBEINVAL\fP
the thread \fIth\fP is already in the detached state

.SH AUTHOR
Xavier Leroy <Xavier.Leroy@inria.fr>

.SH "SEE ALSO"
\fBpthread_create\fP(3),
\fBpthread_join\fP(3),
\fBpthread_attr_setdetachstate\fP(3).